#include "noatunarts.h"

#include <artsflow.h>
#include <stdsynthmodule.h>
#include <convert.h>

//  #include <iostream>
#include <cmath>

using namespace std;
using namespace Arts;

#define DBTOFRAC(x) pow(10,(x)/20.0)

static double lmtr_lvl = 0.5;

/*
 * Limiter function: copied from rvaplugin.c from xmms-normalize plugin, thanks.
 *
 *
 *        / tanh((x + lev) / (1-lev)) * (1-lev) - lev        (for x < -lev)
 *        |
 *   x' = | x                                                (for |x| <= lev)
 *        |
 *        \ tanh((x - lev) / (1-lev)) * (1-lev) + lev        (for x > lev)
 *
 * With limiter level = 0, this is equivalent to a tanh() function;
 * with limiter level = 1, this is equivalent to clipping.
 */
static double limiter(double x)
{
  double xp;

  if (x < -lmtr_lvl)
    xp = tanh((x + lmtr_lvl) / (1-lmtr_lvl)) * (1-lmtr_lvl) - lmtr_lvl;
  else if (x <= lmtr_lvl)
    xp = x;
  else
    xp = tanh((x - lmtr_lvl) / (1-lmtr_lvl)) * (1-lmtr_lvl) + lmtr_lvl;

  return xp;
}

namespace NoatunArts
{

  class Normalize_impl : public Normalize_skel, 
			 public StdSynthModule
  {
  private:
    float g;
    bool enabled;
    

  public:

    Normalize_impl()
      : enabled(false)
    {
    }

    void enable(bool e)
    {
      enabled = e;
//        cout << "enabling " << e << endl;
    }

    bool enable()
    {
      return enabled;
    }

    void gain(float g)
    {
      this->g = DBTOFRAC(g);
//        cout << "setting gain " << this->g << endl;
    }

    float gain()
    {
      return g;
    }

    void calculateBlock(unsigned long samples)
    {
      if (!enabled) {
	memcpy(outleft, inleft, samples * sizeof(float));
	memcpy(outright, inright, samples * sizeof(float));
      }
      else {
	for (unsigned long i = 0; i < samples; i++) {
	  if (g > 1.0) {
	    outleft[i] = limiter(inleft[i] * g);
	    outright[i] = limiter(inright[i] * g);
	  }
	  else {
	    outleft[i] = inleft[i] * g;
	    outright[i] = inright[i] * g;
	  }
	}
      }
    }
  };

  REGISTER_IMPLEMENTATION(Normalize_impl);

}

